@@ -4,9 +4,9 @@ use Carp;
 use DBI;
 use strict;
 use Class::ReturnValue;
-use vars qw($VERSION @ISA $DBIHandle $DEBUG $TRANSCOUNT);
+use vars qw($VERSION @ISA $DBIHandle $DEBUG $TRANSDEPTH);
 
-$TRANSCOUNT = 0;
+$TRANSDEPTH = 0;
 
 $VERSION = '$Version$';
 
@@ -98,7 +98,11 @@ sub Insert {
 
 Takes a paramhash and connects to your DBI datasource. 
 
+You should _always_ set
 
+     DisconnectHandleOnDestroy => 1 
+
+unless you have a legacy app like RT2 or RT 3.0.{0,1,2} that depends on the broken behaviour.
 =cut
 
 sub Connect  {
@@ -112,11 +116,20 @@ sub Connect  {
 	       User => undef,
 	       Password => undef,
 	       RequireSSL => undef,
+           DisconnectHandleOnDestroy => undef,
 	       @_);
 
+    my $dsn = $self->DSN;
+
+    # Setting this actually breaks old RT versions in subtle ways. So we need to explicitly call it
+
+    $self->{'DisconnectHandleOnDestroy'} = $args{'DisconnectHandleOnDestroy'};
+
   $self->BuildDSN(%args);
 
-  my $handle = DBI->connect($self->DSN, $args{'User'}, $args{'Password'}) || croak "Connect Failed $DBI::errstr\n" ;
+    # Only connect if we're not connected to this source already
+   if ((! $self->dbh ) || (!$self->dbh->ping) || ($self->DSN ne $dsn) ) { 
+     my $handle = DBI->connect($self->DSN, $args{'User'}, $args{'Password'}) || croak "Connect Failed $DBI::errstr\n" ;
  
   #databases do case conversion on the name of columns returned. 
   #actually, some databases just ignore case. this smashes it to something consistent 
@@ -124,8 +137,11 @@ sub Connect  {
 
   #Set the handle 
   $self->dbh($handle);
-
   return (1); 
+    }
+
+    return(undef);
+
 }
 # }}}
 
@@ -248,7 +264,11 @@ Disconnect from your DBI datasource
 
 sub Disconnect  {
   my $self = shift;
-  return ($self->dbh->disconnect());
+  if ($self->dbh) {
+      return ($self->dbh->disconnect());
+  } else {
+      return;
+  }
 }
 
 # }}}
@@ -487,6 +507,8 @@ sub BLOBParams {
 
 # }}}
 
+# {{{ DatabaseVersion
+
 =head2 DatabaseVersion
 
 Returns the database's version. The base implementation uses a "SELECT VERSION"
@@ -503,12 +525,10 @@ sub DatabaseVersion {
         $self->{'database_version'}= $vals[0];
     }
 }
-
+# }}}
 
 # {{{ CaseSensitive
 
-
-
 =head2 CaseSensitive
 
 Returns 1 if the current database's searches are case sensitive by default
@@ -524,6 +544,33 @@ sub CaseSensitive {
 
 # }}} 
 
+
+# {{{ _MakeClauseCaseInsensitive
+
+=head2 _MakeClauseCaseInsensitive FIELD OPERATOR VALUE
+
+Takes a field, operator and value. performs the magic necessary to make
+your database treat this clause as case insensitive.
+
+Returns a FIELD OPERATOR VALUE triple.
+
+=cut
+
+sub _MakeClauseCaseInsensitive {
+    my $self = shift;
+    my $field = shift;
+    my $operator = shift;
+    my $value = shift;
+
+    $field = "lower($field)";
+    $value = lc($value);
+
+    return ($field, $operator, $value);
+}
+
+
+# }}}
+
 # {{{ BeginTransaction
 
 =head2 BeginTransaction
@@ -537,9 +584,9 @@ Emulates nested transactions, by keeping a transaction stack depth.
 
 sub BeginTransaction {
     my $self = shift;
-    $TRANSCOUNT++;
-    if ($TRANSCOUNT > 1 ) {
-        return ($TRANSCOUNT);
+    $TRANSDEPTH++;
+    if ($TRANSDEPTH > 1 ) {
+        return ($TRANSDEPTH);
     } else {
        return($self->dbh->begin_work);
     }
@@ -558,13 +605,13 @@ This will turn Autocommit mode back on.
 
 sub Commit {
     my $self = shift;
-    unless ($TRANSCOUNT) {Carp::confess("Attempted to commit a transaction with none in progress")};
-    $TRANSCOUNT--;
+    unless ($TRANSDEPTH) {Carp::confess("Attempted to commit a transaction with none in progress")};
+    $TRANSDEPTH--;
 
-    if ($TRANSCOUNT == 0 ) {
+    if ($TRANSDEPTH == 0 ) {
         return($self->dbh->commit);
     } else { #we're inside a transaction
-        return($TRANSCOUNT);
+        return($TRANSDEPTH);
     }
 }
 
@@ -572,27 +619,61 @@ sub Commit {
 
 # {{{ Rollback
 
-=head2 Rollback
+=head2 Rollback [FORCE]
 
 Tells DBIx::SearchBuilder to abort the current SQL transaction. 
 This will turn Autocommit mode back on.
 
+If this method is passed a true argument, stack depth is blown away and the outermost transaction is rolled back
+
 =cut
 
 sub Rollback {
     my $self = shift;
-    #unless ($TRANSCOUNT) {Carp::confess("Attempted to rollback a transaction with none in progress")};
-    $TRANSCOUNT--;
+    my $force = shift || undef;
+    #unless ($TRANSDEPTH) {Carp::confess("Attempted to rollback a transaction with none in progress")};
+    $TRANSDEPTH--;
 
-    if ($TRANSCOUNT == 0 ) {
+    if ($force) {
+        $TRANSDEPTH = 0;
+       return($self->dbh->rollback);
+    }
+
+    if ($TRANSDEPTH == 0 ) {
        return($self->dbh->rollback);
     } else { #we're inside a transaction
-        return($TRANSCOUNT);
+        return($TRANSDEPTH);
     }
 }
 
 # }}}
 
+=head2 ForceRollback
+
+Force the handle to rollback. Whether or not we're deep in nested transactions
+
+=cut
+
+sub ForceRollback {
+    my $self = shift;
+    $self->Rollback(1);
+}
+
+
+=head2 TransactionDepth
+
+Return the current depth of the faked nested transaction stack.
+
+=cut
+
+sub TransactionDepth {
+    my $self = shift;
+    return ($TRANSDEPTH); 
+}
+
+
+# {{{ ApplyLimits
+
 =head2 ApplyLimits STATEMENTREF ROWS_PER_PAGE FIRST_ROW
 
 takes an SQL SELECT statement and massages it to return ROWS_PER_PAGE starting with FIRST_ROW;
@@ -620,6 +701,12 @@ sub ApplyLimits {
 
 }
 
+
+# }}}
+
+
+# {{{ Join
+
 =head2 Join { Paramhash }
 
 Takes a paramhash of everything Searchbuildler::Record does + a parameter called 'SearchBuilder that contains a ref to a SearchBuilder object'.
@@ -663,7 +750,7 @@ sub Join {
 
 }
 
-
+# }}}
 
 # {{{ DistinctQuery
 
@@ -697,9 +784,7 @@ When we get rid of the Searchbuilder::Handle, we need to disconnect from the dat
   
 sub DESTROY {
   my $self = shift;
-  if( defined $self->dbh ) {
-    $self->Disconnect();
-  }
+  $self->Disconnect if $self->{'DisconnectHandleOnDestroy'};
 }
 
 
